#!/usr/bin/python -t
# -*- coding: UTF8 -*-

## Copyright (C) 2007 Red Hat Inc.
## Written by Barry <mbarabas@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


import gtk 
import gobject

from scheduler.i18n import _
import string
import sys
from guiOutput import fUtils as OfUtils

class PrimitiveObjects:
    
    """
    Class rendering all primitive objects and abstract possibly redundant staff like show function..
    """

    def __exception(self, message, id):
        """
        Simulate throwing new exception with message and id
        Function prints message to stderr and exits with id code

        @param message: exception text
        @type  message: C{string}
        @param id: id
        @type  id: C{integer}
        """
        if id >= 50:
            raise "\nUser Exception: "+message
        else: 
            raise "\nSytem Exception: "+message

    def renderAlignment( self, box, top, bottom, left, right, packing=[False, False] ):
        """
        Render alignment - is using with other primitive-render functions 
        
        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param top: top paddig
        @type  top: C{integer}
        @param right: right padding
        @type  right: C{integer}
        @param bottom: bottom padding
        @type  bottom: C{integer}
        @param left: let padding
        @type  left: C{integer}
        @param packing: packing in box 
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.Alignment
        """

        alignment = gtk.Alignment( 0.5, 0.5, 1.0, 1.0 )
        alignment.set_padding( top, bottom, left, right )
        if box != None: box.pack_start( alignment, packing[0], packing[1], 0 )
        alignment.show( )
    
        return alignment
    
    def renderHBox( self, box=None, align=None, packing=[False, False] ):
        """
        Render horizontal box with alignment

        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param align: array with alignment padding: top, bottom, left, right.
                      For more information see: L{renderAlignment<renderAlignment>}
        @type  align: [C{integer}, C{integer}, C{integer}, C{integer}];
        @param packing: packing in box
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.HBox
        """

        hBox = gtk.HBox( False, 0 )
        
        if align != None:
            (top, bottom, left, right) = align
            alignment = self.renderAlignment( box, top, bottom, left, right )
            alignment.add( hBox )
        elif box != None: box.pack_start( hBox, packing[0], packing[1], 0 )

        hBox.show( )
        if box == None and align != None: return (alignment, hBox)
        else: return hBox
    
    def renderVBox( self, box=None, align=None, packing=[False, False] ):
        """
        Render vertical box with alignment

        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param align: array with alignment padding: top, bottom, left, right.
                      For more information see: L{renderAlignment<renderAlignment>}
        @type  align: [C{integer}, C{integer}, C{integer}, C{integer}];
        @param packing: packing in box
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @rtype:  gtk.VBox
        """
        
        vBox = gtk.VBox( False, 0 )
        
        if align != None:
            (top, bottom, left, right) = align
            alignment = self.renderAlignment( box, top, bottom, left, right )
            alignment.add( vBox )
        elif box != None: box.pack_start( vBox, packing[0], packing[1], 0 )
  
        vBox.show( )
        if box == None and align != None: return (alignment, vBox)
        else: return vBox
 
    def renderVPaned( self, box, box1, box2, align=None, packing=[False, False], packing_top=[False, False], packing_bottom=[False, False] ):
        """
        Render horizontal paned boxes with aligment
        
        @param box: box to be packed in 
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param box1: box in left side of the pane
        @type  box1: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param box2: box in right side of the pane
        @type  box2: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param align: array with alignment padding: top, bottom, left, right.
                      For more information see: L{renderAlignment<renderAlignment>}
        @type  align: [C{integer}, C{integer}, C{integer}, C{integer}];
        @param packing: packing in box 
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @param packing_left: packing left side: resize and shrink 
        @type  packing_left: couple of boolean values [C{boolean}, C{boolean}]
        @param packing_right: packing right side: resize and shrink 
        @type  packing_right: couple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.HPaned
        """

        vpane = gtk.VPaned()

        return self.__renderPaned( vpane, box, box1, box2, align, packing, packing_top, packing_bottom )

    def renderHPaned( self, box, box1, box2, align=None, packing=[False, False], packing_left=[False, False], packing_right=[True, True] ):
        """
        Render horizontal paned boxes with aligment
        
        @param box: box to be packed in 
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param box1: box in left side of the pane
        @type  box1: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param box2: box in right side of the pane
        @type  box2: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param align: array with alignment padding: top, bottom, left, right.
                      For more information see: L{renderAlignment<renderAlignment>}
        @type  align: [C{integer}, C{integer}, C{integer}, C{integer}];
        @param packing: packing in box 
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @param packing_left: packing left side: resize and shrink 
        @type  packing_left: couple of boolean values [C{boolean}, C{boolean}]
        @param packing_right: packing right side: resize and shrink 
        @type  packing_right: couple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.HPaned
        """

        hpane = gtk.HPaned( )

        return self.__renderPaned( hpane, box, box1, box2, align, packing, packing_left, packing_right )

    def __renderPaned( self, pane, box, box1, box2, align, packing, packing_left, packing_right ):
        """
        Render horizontal paned boxes with aligment
        
        @param box: box to be packed in 
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param box1: box in left side of the pane
        @type  box1: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param box2: box in right side of the pane
        @type  box2: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param align: array with alignment padding: top, bottom, left, right.
                      For more information see: L{renderAlignment<renderAlignment>}
        @type  align: [C{integer}, C{integer}, C{integer}, C{integer}];
        @param packing: packing in box 
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @param packing_left: packing left side: resize and shrink 
        @type  packing_left: couple of boolean values [C{boolean}, C{boolean}]
        @param packing_right: packing right side: resize and shrink 
        @type  packing_right: couple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.HPaned
        """

        if box1 == None or box2 == None:
            self.__exception("Bad calling renderHPaned(): missing gtk.Box object", 50)

        if align != None:
            (top, bottom, left, right) = align
            alignment = self.renderAlignment( box, top, bottom, left, right, packing=[True, True] )
            alignment.add( pane )
        elif box != None: box.pack_start( pane, packing[0], packing[1], 0 )

        pane.pack1( box1, resize=packing_left[0], shrink=packing_left[1] )
        pane.pack2( box2, resize=packing_right[0], shrink=packing_right[1] )

        pane.show( )
        return pane

    def renderComboBox( self, box=None, callbacks={}, list=[], packing=[True, True]):
        """
        Render ComboBox with callback function on changed signal

        @param box: box to be packed to 
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param callbacks: dictionary of signals and functions to call when comboBox
        @type  callbacks: C{dictionary}
        @param list: list of combo items
        @type  list: C{list}
        @param packing: packing in box
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.ComboBox
        """

        combo = gtk.combo_box_new_text( )
        combo.show()
        for item in list:
            combo.append_text( item )
        combo.set_active( 0 )

        for key in callbacks.keys():
            combo.connect( key, callbacks[key], combo )
        if box != None: box.pack_start( combo, packing[0], packing[1], 0 )
        
        return combo

    def renderTreeView( self, box, model, callbacks={}, selection_type=gtk.SELECTION_MULTIPLE, enable_tree_lines=False, packing=[True, True], grid_lines=None ):
        """
        Render TreeView with callback function on toggle row

        @param box: box to be packed to [gtk.Box or None]
        @type box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param model: TreeView model
        @type  model: C{gtk.TreeModel}
        @param callbacks: dictionary of sigals and functions to call
        @type  callbacks: C{dictionary}
        @param selection_type: selection type
        @type  selection_type: C{gtk.SELECTION_*}
        @param enable_tree_lines: enabling treeView lines
        @type  enable_tree_lines: C{boolean}
        @param packing: packing in box
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.TreeView
        """

        if model == None:
            self.__exception("Bad renderTreeView ", 50)
        treeView = gtk.TreeView( model )
        
        selection = treeView.get_selection( )
        selection.set_mode( selection_type )
 
        for key in callbacks.keys():
            selection.connect( key, callbacks[key], treeView ) 

        treeView.set_enable_tree_lines( enable_tree_lines )        
        if enable_tree_lines: treeView.set_grid_lines( gtk.TREE_VIEW_GRID_LINES_HORIZONTAL )
        treeView.show( )
        
        if gtk.gtk_version[0] == 2 and gtk.gtk_version[1] > 10:
            if grid_lines != None: treeView.set_grid_lines( grid_lines )
            
        if box != None:
            if type( box ) in [ type( gtk.ScrolledWindow())  ]:
                box.add( treeView )
            else: box.pack_start( treeView, packing[0], packing[1], 0 )
        
        return treeView

    def renderTreeViewColumnText( self, treeView, label, id, expand=False, sort=False, visible=True, markup=True, callbacks={} ):
        """
        Render label to TreeView as TreeViewColumn 

        @param treeView: TreeView to render column to
        @type  treeView: C{gtk.TreeView}
        @param label: text label
        @type  label: C{string}
        @param id: position in columns
        @type  id: C{inteager}
        @param expand: if column should expand if there is some free place
        @type  expand: C{boolean}
        @param sort: if column should be sortable
        @type  sort: C{boolean}
        @param visible: if column is visible by default
        @type  visible: C{boolean}
        @param markup: if using markup in label is enabled
        @type  markup: C{boolean}
        @param callbacks: dictionary of signals and functions
        @type  callbacks: C{dictionary}
        @return: If parameter C{treeView} is not C{None} it returns nothing

             >>> if treeView != None: treeView.append_column( column )
             else: return column

        @rtype: gtk.TreeViewColumn
        @note: For every signal is sending id of column  
        """
        
        cell = gtk.CellRendererText( )
        cell.set_property( 'cell-background', 'white' )
        for key in callbacks.keys():
            cell.connect( key, callbacks[key], id)
        if markup: column = gtk.TreeViewColumn( label, cell, markup=id )
        else:      column = gtk.TreeViewColumn( label, cell, text=id )
        if sort: 
            column.set_sort_indicator( True )
            column.set_sort_column_id( id )
        column.set_expand( expand )
        column.set_visible( visible )
        
        if treeView != None: treeView.append_column( column )
        else: return column
        
    def renderTreeViewColumnToggle( self, treeView, label, id, callbacks={}, ):
        """
        Render toggle button to TreeView as TreeViewColumn 

        @param treeView: TreeView to render column to
        @type  treeView: C{gtk.TreeView}
        @param label: text label
        @type  label: C{string}
        @param id: position in columns
        @type  id: C{integer}
        @param callbacks: dictionary of signals and functions
        @type  callbacks: C{dictionary}
        @return: If parameter C{treeView} is not C{None} it returns nothing

             >>> if treeView != None: treeView.append_column( column )
             else: return column

        @rtype: gtk.TreeViewColumn
        @note: For every signal is sending id of column
        """
        
        cell = gtk.CellRendererToggle()
        cell.set_property('activatable', True)
        for key in callbacks.keys(): 
            cell.connect( key, callbacks[key], treeView.get_model(), id)
            
        column = gtk.TreeViewColumn( label, cell )
        column.add_attribute(cell, "active", id)
        if treeView != None: treeView.append_column( column )
        else: return column
            
    def renderTreeViewColumnPixbuf( self, treeView, label, id, max_width=32 ):
        """
        Render picture to TreeView as TreeViewColumn

        @param treeView: TreeView to render column to
        @type  treeView: C{gtk.TreeView}
        @param label: text label
        @type  label: C{string}
        @param id: position in columns
        @type  id: C{integer}
        @param max_width: value of maximal width of column
        @type  max_width: C{integer}
        @return: If parameter C{treeView} is not C{None} it returns nothing

             >>> if treeView != None: treeView.append_column( column )
             else: return column

        @rtype: gtk.TreeViewColumn
        """
        
        cell = gtk.CellRendererPixbuf( )
        cell.set_property( 'cell-background', 'white' )
        column = gtk.TreeViewColumn( label, cell )
        column.set_max_width( max_width )
        column.set_attributes(cell, pixbuf=id)
        
        if treeView != None: treeView.append_column( column )
        else: return column

    def renderNoteBook( self, box=None, tab_position=gtk.POS_TOP, packing=[True,True] ):
        """
        Render base staff of notebook. After this, please call renderNBPage for every new page in NB

        @param box: box to be packed to [gtk.Box]
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param tab_position: position of tabs
        @type  tab_position: C{gtk.POS_TOP} or C{gtk.POS_BOTTOM} or ...
        @param packing: packing in box
        @type  packing: couple of booleans [C{boolean}, C{boolean}]
        @rtype: gtk.Notebook
        """

        noteBook = gtk.Notebook( )
        noteBook.set_tab_pos( tab_position )
        noteBook.show( )
 
        if box != None: box.pack_start( noteBook, packing[0], packing[1], 0 )

        return noteBook

    def renderNBPage( self, noteBook, pageLabel, with_close_button=False, markup=True ):
        """
        This function is called after renderNoteBook func for every new page.

        @param noteBook: where will be page added
        @type  noteBook: C{gtk.Notebook}
        @param pageLabel: text label of the page
        @type  pageLabel: C{string}
        @param with_close_button: if you want close button next to the page label 
        @type  with_close_button: C{boolean}
        @param markup: if using markup in label is enabled
        @type  markup: C{boolean}
        @return: gtk.VBox which is inside the new tab
        @rtype: gtk.VBox
        """

        box = self.renderVBox( packing=[True, True] )
        box.show( ) 

        label = gtk.Label( pageLabel )
        label.set_use_markup( markup )
        label.show( )

        if with_close_button:
            iconw = self.setIconStock( gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU )
            button = gtk.Button( )
            button.set_relief( gtk.RELIEF_NONE )
            button.set_image( iconw )
            button.show( )
            labelBox = self.renderHBox()
            labelBox.pack_start( label, True, True, 0 )
            labelBox.pack_start( button, False, False, 0 )
            noteBook.append_page( box, labelBox )
        else: noteBook.append_page( box, label )

        return box

    def renderScrolledWindow( self, box, inbox, hpolicy, vpolicy, shadow, place ):
        """
        Render scrolled window with inbox within

        @param box: box to pack window in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or {None}
        @param inbox: inner box of window
        @type  inbox: C{gtk.VBox} or C{gtk.HBox} or {None}
        @param hpolicy: horisontal policy
        @type  hpolicy: C{gtk.POLICY*}
        @param vpolicy: vertical policy
        @type  vpolicy: C{gtk.POLICY}
        @param shadow: shadow of window
        @type  shadow: C{gtk.SHADOW*}
        @param place: placement of window
        @rtype: gtk.ScrolledWindow
        @todo [1.2]: make default values
        """

        scWindow = gtk.ScrolledWindow( )
        scWindow.set_policy( hpolicy, vpolicy )
        scWindow.set_shadow_type( shadow )
        scWindow.set_placement( place )
        # hack to silent Warnings
        if type( inbox ) in [ type( gtk.VBox() ), type( gtk.Notebook() ), type( gtk.Fixed() ) ]: 
            scWindow.add_with_viewport( inbox )
            inbox.get_parent().set_shadow_type( shadow )
        else: scWindow.add( inbox)
        
        scWindow.show( )
        
        if box != None: box.pack_start( scWindow, True, True, 0 )
        return scWindow

    def renderMenuBar( self, box=None, packing=[False, False] ):
        """ 
        Render top menu bar

        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or {None}
        @param packing: packing in box
        @type  packing: list of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.MenuBar
        """

        menuBar = gtk.MenuBar( )
        menuBar.show( )
        if box != None: box.pack_start( menuBar, packing[0], packing[1], 0 )
        return menuBar

    def renderMenuItem( self, label, prefix=None, callbacks={}, gobject=None, Note=None, enabled=True ):
        """ 
        Render menu item with callback wich is called after recieved signal

        @param label: text label
        @type  label: C{string}
        @param prefix: prefix for keyboard interaction
        @type  prefix: C{string}
        @param callbacks: dictionary of signals and functions
        @type  callbacks: C{dictionary}
        @param gobject: graphical object - picture
        @type  gobject: C{gtk.Object}
        @param Note: some hints
        @type  Note: C{string}
        @param enabled: if button is enabled
        @type  enabled: C{boolean}
        @rtype: gtk.MenuItem
        """

        action = gtk.Action( label, prefix, Note, gobject)
        action.set_property('short-label', prefix)

        for key in callbacks.keys(): 
            action.connect( key, callbacks[key] )

        # create menu item
        item = action.create_menu_item()
        item.set_sensitive( enabled )
        item.show( )
        
        return item
    
    def renderCheckMenuItem( self, label ):
        """ 
        Render checkButton as menuItem 

        @param label: label of checkBox
        @type  label: C{string}
        @rtype: gtk.CheckMenuItem
        """

        # create menu item
        item = gtk.CheckMenuItem( label )
        item.show( )
        
        return item
    
    def renderRootMenuItem( self, label, prefix=None, menuBar=None, submenu=[] ):
        """ 
        Create root menu item with submenu

        @param label: text label
        @type  label: C{string}
        @param prefix: prefix for keyboard interaction
        @type  prefix: C{string}
        @param menuBar: to add this item into
        @type  menuBar: C{gtk.MenuBar}
        @param submenu: list of items which will make the submenu of this item
        @type  submenu: C{list}
        @rtype: gtk.MenuItem
        """

        item = self.renderMenuItem( label, prefix )
        
        if submenu != []:
            file_menu = gtk.Menu( )
            for subitem in submenu: file_menu.append( subitem )
            item.set_submenu( file_menu )
            
        if menuBar != None: menuBar.append( item )
        return item

    def renderToolBar( self, box=None, style=gtk.TOOLBAR_BOTH, tooltips=True, orientation=gtk.ORIENTATION_HORIZONTAL, border=0, packing=[False, False] ):
        """ 
        Render Tool Bar

        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or {None}
        @param style: style of toolBar
        @type  style: C{gtk.TOOLBAR*}
        @param tooltips: if tooltips will be shown
        @type  tooltips: C{boolean}
        @param orientation: orientation of toolBar
        @type  orientation: {gtk.ORIENTATION*}
        @param border: border width
        @type  border: C{integer}
        @param packing: packing in box
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.Toolbar
        """

        toolbar = gtk.Toolbar( )
        toolbar.set_style( style )
        toolbar.set_tooltips( tooltips )
        toolbar.set_orientation( orientation )
        toolbar.set_border_width( border )
        toolbar.show( )
        
        if box != None: box.pack_start( toolbar, packing[0], packing[1], 0 )
        return toolbar

    def setIconStock( self, icon, size=gtk.ICON_SIZE_SMALL_TOOLBAR ):
        """
        Render Image from STOCK icon

        @param icon: stock icon
        @type  icon: C{string} - path to icon file
        @param size: size of the image
        @type  size: C{gtk.ICON_SIZE_*}
        @rtype: gtk.Image
        """
        
        iconw = gtk.Image( )
        iconw.show()
        iconw.set_from_stock( icon, size )
        return iconw
    
    def renderEventBox( self, box=None, bg_color=None, packing=[False, False], events=None, callbacks={}, border=0 ):
        """
        Render container object gtk.EventBox

        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param bg_color: color of eventBox background
        @type  bg_color: C{string}
        @param packing: packing in box
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @param events: events to register
        @type  events: C{gtk events}
        @param callbacks: dictionary of signals and functions [dictionary]
        @type  callbacks: C{dictionary}
        @param border: border width
        @type  border: C{integer}
        @rtype: gtk.EventBox
        """
        
        eventBox = gtk.EventBox( )
        if events != None: eventBox.set_events ( events )
        eventBox.set_border_width( border )
        if bg_color != None: eventBox.modify_bg(gtk.STATE_NORMAL, eventBox.get_colormap().alloc_color(bg_color))
        for key in callbacks: 
            eventBox.connect( key, callbacks[key] )
        eventBox.show( )
        
        if box != None: box.pack_start( eventBox, packing[0], packing[1], 0 )
        return eventBox

    def renderOutputButtonArea( self, box ):
        """
        Render area under the output buttons, add some options here

        @param box: box to add to
        @type  box: C{gtk.Window*}
        @rtype: gtk.VBox
        @note: This is for renderVBox when box as first parameter have to be added by func add not pack_start
        """

        if box == None:
            self.__exception("Bad using renderOutputButtonArea(): missing box to add to gtk.VBox", 50)
        
        box1 = self.renderVBox( )
        box.add( box1 )
        return box1
        
    def appendMessage( self, model, message):
        """
        Append message to outputView

        @param model: TreeView model
        @type  model: C{gtk.TreeView}
        @param message: message to append
        @todo [1.2]: move to formatter
        """

        if model == None:
            self.__exception(_("Wrong call of appendMessage() function: missing gtk.TreeModel"), 50)
        if message == None:
            self.__exception(_("Wrong call of appendMessage() function: missing message string"), 50)
            
        model.get_model().append( message )

    def renderLabel( self, box=None, label="", align=None, markup=True, line_wrap_mode=gtk.WRAP_WORD, packing=[False, False], padding=[0,0] ):
        """
        Render simple label

        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param label: text label
        @type  label: C{string}
        @param align: list of align properties
        @type  align: list of integers [C{integer}, C{integer}, C{integer}, C{integer}]
        @param markup: if using markup is enabled
        @type  markup: C{boolean}
        @param line_wrap_mode: if text should be wrapped
        @type  line_wrap_mode: C{gtk.WRAP*}
        @param packing: packing for box
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @param padding: padding of label
        @type  padding: couple of integer values [C{integer}, C{integer}]
        @return: If align is not None, C{gtk.Alignment}, C{gtk.Label}, else C{gtk.Label}
        @rtype: gtk.Label
        """

        if label == None: 
            self.__exception("Wrong call of renderLabel(): expected string in label got None !", 50)
            
    	label = gtk.Label( label )
    	label.set_use_markup( markup )
        label.set_line_wrap( line_wrap_mode != gtk.WRAP_NONE )
    	label.set_line_wrap_mode( line_wrap_mode )
        label.set_alignment( 0.0, 0.0 )
    	label.set_padding( padding[0], padding[1] )
    	label.show( )
        
    	if align != None:
            [top, bottom, left, right] = align
            alignment = self.renderAlignment( box, top, bottom, left, right, [False, False] )
            alignment.add( label )
            return alignment, label
        elif box != None:
            box.pack_start( label, packing[0], packing[1], 0 )
	
        return label

    def renderTextView( self, box=None, wrap_mode=gtk.WRAP_NONE, justify=gtk.JUSTIFY_LEFT, packing=[False, False], editable=False, accepts_tab=False ):
        """
        Render TextView 

        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param wrap_mode: mdoe of wrapping text when reached the end of line
        @type  wrap_mode: C{gtk.WRAP*}
        @param justify: justify of text
        @type  justify: C{gtk.JUSTIFY*}
        @param packing: packing for box
        @type  packing: couple of boolean values [C{boolean}, C{boolean}]
        @param editable: set the text to be editable or not
        @type  editable: C{boolean}
        @param accepts_tab: if textView accepts <TAB>
        @type  accepts_tab: C{boolean}
        @rtype: gtk.TextView
        """
        
        buffer = gtk.TextBuffer( )
        textView = gtk.TextView( buffer )
        textView.set_wrap_mode( wrap_mode )
        textView.set_justification( justify )
        textView.set_editable( editable )
        textView.set_accepts_tab( accepts_tab )
        textView.show( )
        
        if box != None: box.pack_start( textView, packing[0], packing[1], 0 )
        return textView
        
    def __renderSeparator( self, separator=None, box=None, align=None, packing=[True, True] ):
        """ 
        Render visible separator. Do not use this methode for rendering separator. 
        Use methods L{renderHSeparator<renderHSeparator>} or L{renderVSeparator<renderVSeparator>}

        @param separator: separator from renderHSpeparator or renderVSeparator
        @type  separator: C{gtk.VSeparator} or C{gtk.HSeparator}
        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param align: list of align properties
        @type  align: list of integers [C{integer}, C{integer}, C{integer}, C{integer}]
        @param packing: packing for box
        @type  packing: ouple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.HSeparator or gtk.VSeparator
        """
        if separator == None:
            self.__exception("Wrong call of renderSeparator() function. You have to call renderHSeparator or renderVSeparator.", 51)
        if type(separator) != type(gtk.HSeparator()) and type(separator) != type(gtk.VSeparator()): 
            self.__exception("Wrong call of renderSeparator() function: \n  Wrong type of separator: "+
                             `type(separator)`+
                             "\n  Expected: "+`type(gtk.HSeparator())`+" or "+`type(gtk.VSeparator())`+"\n", 51)
        
        if align != None:
            top, bottom, left, right = align
            alignment = self.renderAlignment( None, top, bottom, left, right )
            alignment.add( separator )
            if box != None: box.pack_start( alignment, packing[0], packing[1], 0 ) 
            return alignment, separator
        elif box != None: box.pack_start( separator, packing[0], packing[1], 0 ) 

        return separator

    def renderHSeparator( self, box=None, align=None, packing=[True,True] ):
        """
        Render horisontal separator

        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param align: list of align properties
        @type  align: list of integers [C{integer}, C{integer}, C{integer}, C{integer}]
        @param packing: packing for box
        @type  packing: ouple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.HSeparator
        """
        separator = gtk.HSeparator( )
        separator.show( )
        return self.__renderSeparator( separator, box, align, packing )
        
    def renderVSeparator( self, box=None, align=None, packing=[True,True] ):
        """
        Render vertical separator

        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param align: list of align properties
        @type  align: list of integers [C{integer}, C{integer}, C{integer}, C{integer}]
        @param packing: packing for box
        @type  packing: ouple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.HSeparator
        """
        separator = gtk.VSeparator( )
        separator.show( )
        return self.__renderSeparator( separator, box, align, packing )
     
    def renderProgressBar( self, box=None, align=None, packing=[True, True], label="" ):
        """
        Render ProgressBar

        @param box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param align: list of align properties
        @type  align: list of integers [C{integer}, C{integer}, C{integer}, C{integer}]
        @param packing: packing for box
        @type  packing: ouple of boolean values [C{boolean}, C{boolean}]
        @param label: text inside the ProgressBar
        @type  label: C{string}
        @rtype: gtk.ProgressBar
        """

        progressBar = gtk.ProgressBar()
        progressBar.set_text("")
        progressBar.queue_resize()
        progressBar.show()
        
        if align != None:
            top,  bottom, left, right = align
            alignment = self.renderAlignment( None, top, bottom, left, right )
            alignment.add( progressBar )
            if box != None: box.pack_start( alignment, packing[0], packing[1], 0 ) 
            return alignment, progressBar
        elif box != None: box.pack_start( progressBar, packing[0], packing[1], 0 )
        
        return progressBar
        
    def renderAnimation( self, box, path, align=None, packing=[False, False] ):
        """
        Render Image (Animation - GIF)

        @param  box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param path: path to image file
        @type  path: C{string}
        @param align: list of align properties
        @type  align: list of integers [C{integer}, C{integer}, C{integer}, C{integer}]
        @param packing: packing for box
        @type  packing: ouple of boolean values [C{boolean}, C{boolean}]
        @rtype: gtk.Image
        """
        if path == None: 
            self.__exception("Wrong call of renderAnimation() function: missing image path", 50)
        
        image = gtk.Image( )
        animation = gtk.gdk.PixbufAnimation( path )
        image.set_from_animation( animation )
        image.show( )
 
        if align != None:
            top, bottom, left, right = align
            alignment = self.renderAlignment( None, top, bottom, left, right, [False, False] )
            alignment.add( image )
            if box != None: box.pack_start( alignment, packing[0], packing[1], 0 )
        elif box != None: box.pack_start( image, packing[0], packing[1], 0 )

        return image

    def renderWindow( self, width, height, resizable=True, type=gtk.WINDOW_TOPLEVEL ):
        """
        Render Window

        @param width: width of window
        @type  width: C{integer}
        @param height: height of window
        @type  height: C{integer}
        @param resizable: if window will be resizable
        @type  resizable: C{boolean}
        @param type: type of window
        @type  type: C{gtk.WINDOW_*}
        @rtype: gtk.Window
        """
        
        window = gtk.Window( type )
        window.resize( width, height )
        window.set_default_size( width, height )
        window.set_resizable( resizable )
        
        return window

    def renderButton( self, box, type=None, icon=None, size=gtk.ICON_SIZE_MENU, packing=[False, False], align=None, callbacks=None ):
        """
        Render Button or ToggleButton or RadioButton 

        @param  box: box to be packed in
        @type  box: C{gtk.VBox} or C{gtk.HBox} or C{None}
        @param type: type of button 
        @type  type: C{toggle} | C{radio}
        @param icon: icon of button
        @type  icon: C{gtk.Image} or C{str} for stock
        @param size: size of the image
        @type  size: C{gtk.ICON_SIZE_*}
        @param packing: packing for box
        @type  packing: ouple of boolean values [C{boolean}, C{boolean}]
        @param align: list of align properties
        @type  align: list of integers [C{integer}, C{integer}, C{integer}, C{integer}]
        @param callbacks: dictionary of signals and functions [dictionary]
        @type  callbacks: C{dictionary}
        @rtype: gtk.Button or gtk.RadioButton or gtk.ToggleButton
        """
        if type == "toggle": 
            button = gtk.ToggleButton( )
        elif type == "radio":
            button = gtk.RadioButton( )
        else: button = gtk.Button( )

        if icon != None:
            iconw = self.setIconStock( icon , size  )
            button.set_image( iconw )
        button.set_relief( gtk.RELIEF_NONE )
        button.show( )

        for key in callbacks: 
            button.connect( key, callbacks[key] )

        if align != None:
            top, bottom, left, right = align
            alignment = self.renderAlignment( None, top, bottom, left, right, [False, False] )
            alignment.add( button )
            if box != None: box.pack_start( alignment, packing[0], packing[1], False )
        elif box != None: box.pack_start( button, packing[0], packing[1], False )

        return button
    
    def renderToolButton( self, label, icon=None, callbacks=None, sensitive=True, type=None ):
        """
        Render Button or ToggleButton or RadioButton as ToolButtons

        @param label: the button label
        @type  label: C{string}
        @param icon: string (gtk.STOCK*) of button icon 
        @type  icon: C{gtk.STOCK*}
        @param callbacks: dictionary of callbacks with signals as keys and function pointers as data
        @type  callbacks: dictionary
        @param sensitive: sensitivity of button (optional)
        @type  sensitive: C{boolean}
        @param type: type of button ("Toggle",..) - do not use by default. Better to use:
            >>> renderToggleToolButton( label, icon, callbacks, .. )
            >>> renderRadioToolButton( label, icon, callbacks, .. )
        @type  type: specific C{string}
        """

        if type == "Toggle": toolButton = gtk.ToggleToolButton( ) 
        else: toolButton = gtk.ToolButton( ) 

        if icon != None: toolButton.set_icon_widget( self.setIconStock( icon ) )
        toolButton.set_label( label )
        for key in callbacks.keys(): 
            toolButton.connect( key, callbacks[key] )
        toolButton.set_sensitive( sensitive )

        toolButton.show()

        return toolButton

    def renderToggleToolButton( self, label, icon=None, callbacks=None, sensitive=True ):
        """
        Render ToggleButton as ToolButton
        Calling function renderToolButton

        @param label: the button label
        @type  label: C{string}
        @param icon: string (gtk.STOCK*) of button icon 
        @type  icon: C{gtk.STOCK*}
        @param callbacks: dictionary of callbacks with signals as keys and function pointers as data
        @type  callbacks: dictionary
        @param sensitive: sensitivity of button (optional)
        @type  sensitive: C{boolean}
        """

        return self.renderToolButton( label, icon, callbacks, sensitive, type="Toggle" )

class RenderingObject( PrimitiveObjects ):

    def __init__( self ):
        
        self.pobject = PrimitiveObjects( )
        self.fUtils = OfUtils()
        
class RenderObjects( PrimitiveObjects ):
    """
    Class that abstract primitive objects and take care of every rendering in project
    """

    def __init__( self, handler, formatter ):
        """
        Initializing class with handler parameter which is interface to signalHandler

        @param handler: Object src.SignalHandler stored into C{self.handler}
        @param formatter: Object src.OutputFormatter stored into C{formatter}

        Enabling tooltips in C{self.tooltips}
        """

        self.handler = handler
        self.formatter = formatter
        self.tooltips = gtk.Tooltips()
        self.tooltips.enable( )
        self.hintWindow = None #: workaround to disable Warning 

    def TestView( self, OutputArea=None ):
        """
        Rendering TreeView of tests

        @param box: box to pack in
        @type  box: C{gtk.HBox} or C{gtk.VBox}
        """
        self.OTestList = TestList( self.handler, OutputArea )

    def MainPane( self, box, align=None ):
        """
        Rendering pane on wich left side is "menu" with filter and testView and right side with MainBook

        @param box: box to pack in
        @type  box: C{gtk.HBox} or C{gtk.VBox}
        @param align: array with alignment padding: top, right, bottom, left.
                      For more information see: L{renderAlignment<renderAlignment>}
        @type  align: [C{integer}, C{integer}, C{integer}, C{integer}];
        """

        #assert box == None, "RenderObject.MainPane.box is None" #: debug box should not be None

        menuBox = self.renderVBox( packing=[True, True] ) # Box with menu and filter
        mainBox = self.renderVBox( packing=[True, True] ) # Box with main view

        scWindow1 = self.renderScrolledWindow( None, menuBox, gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC, gtk.SHADOW_NONE, gtk.CORNER_TOP_LEFT  )

        pane = self.renderHPaned( box, scWindow1, mainBox, align )

        return menuBox, mainBox

    def MenuBar( self, OutputArea=None ):
        """
        Menu bar.. File, View, Edit, Help.. 

        @param box: box to pack in
        @type  box: C{gtk.HBox} or C{gtk.VBox}
        """
        
        self.OMenuBar = MenuBar( self.handler, OutputArea )
    
    def ToolBar( self, OutputArea=None ):
        """
        Bar with large buttons like Start, Stop, Quit, ...

        @param box: box to pack in
        @type  box: C{gtk.HBox} or C{gtk.VBox}
        """

        self.OToolBar = ToolBar( self.handler, OutputArea )

    def MainBook( self, box=None ):
        """
        Render main frame with output, descriptions, .. 

        @param box: box to pack in
        @type  box: C{gtk.HBox} or C{gtk.VBox}
        """

        self.mainBook = self.renderNoteBook( box )
        self.mainBook.connect( "switch-page", self.handler.get_callback("NB_SWITCH_PAGE") )
        self.mainPage = self.renderNBPage( self.mainBook, _("Overview") )
        self.resultPage = self.renderNBPage( self.mainBook, _("Test Results") )

        ############ Main Page
        self.mainPageArea = self.renderOutputButtonArea( self.mainPage )

        mainBox = gtk.Fixed( )
        mainBox.show( )

        self.mainPageSeparators = list()
        self.resultPageSeparators = list()

        sc = self.renderScrolledWindow( self.mainPageArea, mainBox, gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC, gtk.SHADOW_NONE, gtk.CORNER_TOP_LEFT )
 
        base = 40
        info = base+80
        groups = info+60
        levels = groups+60

        label                = self.renderLabel( None, "<b>"+_("Test info")+"</b>", packing=[False, False] )
        separator            = self.renderHSeparator( None )
        self.testInfo = self.renderLabel( None, " ", packing=[False, False] )
        separator.set_size_request( 500, 5 )
        self.mainPageSeparators.append( separator )
        mainBox.put( label, 20, base )
        mainBox.put( separator, 20, base+20 )
        mainBox.put( self.testInfo, 30, base+30 )

        label                = self.renderLabel( None, "<b>"+_("Test groups")+"</b>", packing=[False, False] )
        separator            = self.renderHSeparator( None )
        self.testGroups = self.renderLabel( None, " ", packing=[False, False] )
        separator.set_size_request( 500, 5 )
        self.mainPageSeparators.append( separator )
        mainBox.put( label, 20, info )
        mainBox.put( separator, 20, info+20 )
        mainBox.put( self.testGroups, 30, info+30 )

        label                = self.renderLabel( None, "<b>"+_("Test is defined in levels")+"</b>", packing=[False, False] )
        separator            = self.renderHSeparator( None )
        self.testLevels = self.renderLabel( None, " ", packing=[False, False] )
        separator.set_size_request( 500, 5 )
        self.mainPageSeparators.append( separator )
        mainBox.put( label, 20, groups )
        mainBox.put( separator, 20, groups+20 )
        mainBox.put( self.testLevels, 30, groups+30 )

        label                = self.renderLabel( None, "<b>"+_("Test description")+"</b>", packing=[True, True] )
        separator            = self.renderHSeparator( None )
        self.testDescription = self.renderLabel( None, " ", packing=[False, False] )
        self.testDescription.set_size_request(500, -1)
        self.testDescription.set_alignment(0,0)
        separator.set_size_request( 500, 5 )
        self.mainPageSeparators.append( separator )
        mainBox.put( label, 20, levels )
        mainBox.put( separator, 20, levels+20 )
        mainBox.put( self.testDescription, 30, levels+30 )

        self.outputTopArea = self.renderHBox( )
        self.outputBottomArea = self.renderVBox( )

        self.renderVPaned( self.resultPage, self.outputTopArea, self.outputBottomArea, packing=[True, True] )

        ############ Review box 
        self.OOutputTestList = OutputTestList( self.handler, self.outputTopArea )
        
        ############ Summary box
        self.OOutputTestSummary = OutputTestSummary( self.handler, self.outputTopArea )

        ############ Test results box
        
        self.OOutputTestResults = OutputTestResults( self.handler, self.outputBottomArea )
        
    def SimpleFilter( self, box=None ):
        """
        Simple filter on the left side of application above treeView with tests

        @param box: box to pack in
        @type  box: C{gtk.HBox} or C{gtk.VBox}
        """

        #assert box == None, "RenderObject.SimpleFilter.box is None" #: debug box should not be None

        box = self.renderHBox( box, (5, 5, 5, 5) )

        self.group_filter = self.renderComboBox( box, { "changed": self.handler.get_callback("FILTER_GROUP")}, 
                                [   _('All'),
                                    ],
                                )
        
        self.level_filter = self.renderComboBox( box, { "changed": self.handler.get_callback("FILTER_LEVEL")},
                                [   _('Naive')+" (1)",
                                    _('Desktop')+" (2)",
                                    _('Network')+" (3)",
                                    _('Server')+" (4)",
                                    _('Paranoid')+" (5)",
                                    ],
                                )
        self.tooltips.set_tip(self.group_filter, _("Choose a group to filter tests"), "Private")
        self.tooltips.set_tip(self.level_filter, _("Choose a level to run the test"), "Private")

    def RootBox( self, window=None ):
        """
        At the begining was Root vertical Box and other staff are being rendered

        @param window: main window
        @type  window: C{gtk.Window}
        """

        box = self.renderVBox( )
        box.show( )
        if window != None: window.add( box )
        return box
    
    def TestInfoBox( self, window ):
        """
        Box with information about selected message

        @param window: main window
        @type  window: C{gtk.Window}
        """

        WIDTH = 100
        HEIGHT = 30
        
        x = window.get_position( )[0] + window.get_size()[0]
        y = window.get_position( )[1] + 200
        
        self.hintWindow = self.renderWindow(WIDTH, HEIGHT, False, gtk.WINDOW_POPUP)
        
        box = self.renderVBox( None )
        
        self.renderHSeparator( box, [5, 5, 5, 5] )
        align, self.hintLabel = self.renderLabel(box, "", [5,10,10,5], True, gtk.WRAP_WORD, [True, True])
        self.renderHSeparator( box, [5, 5, 5, 5] )

        callbacks = { "button-press-event": self.handler.get_callback("HINT_WINDOW_CLOSE") }
        eventBox = self.renderEventBox(None, "#FFFBD8", None, gtk.gdk.BUTTON_PRESS_MASK, callbacks, 1)
        #button.connect( "pressed", callback_close, None )
        eventBox.add( box )

        self.hintWindow.add( eventBox )
        self.hintWindow.move( x, y )  

    def StatusBar( self, OutputArea ):
        """
        Creates status bar at the bottom of main window

        @param box: box to pack in
        @type  box: C{gtk.HBox} or C{gtk.VBox}
        """
        
        self.OStatusWindow = StatusWindow( self.handler, OutputArea )
 
    def OpenDialog( self ):
        """
        Create dialog to choose a file
        """

        file_open = gtk.FileChooserDialog(title="Select file to save.."
                                    , action=gtk.FILE_CHOOSER_ACTION_SAVE
                                    , buttons=(gtk.STOCK_CANCEL
                                    , gtk.RESPONSE_CANCEL
                                    , gtk.STOCK_OK
                                    , gtk.RESPONSE_OK))
           
        #: Create and add the 'all files' filter
        filter = gtk.FileFilter()
        filter.set_name("All files")
        filter.add_pattern("*")
        file_open.add_filter(filter)
            
        #: Init the return value
        result = ""
        if file_open.run() == gtk.RESPONSE_OK:
            result = file_open.get_filename()
        
        file_open.destroy( )
        if result != "": return result


class OutputTestSummary( RenderingObject ):
    
    def __init__( self, handler, OutputArea ):
        
        RenderingObject.__init__( self )
        
        self.pobject.renderVSeparator( OutputArea, packing=[False, False] )

        self.outputInfoBox = gtk.Fixed()
        self.outputInfoBox.show( )
        self.pobject.renderScrolledWindow( OutputArea, self.outputInfoBox, gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC, gtk.SHADOW_NONE, gtk.CORNER_TOP_LEFT )

        base = 10
        label                   = self.pobject.renderLabel( None, "<b>"+_("Summary")+"</b>", packing=[False, False] )
        separator               = self.pobject.renderHSeparator( None )
        self.outputTestSummary   = self.pobject.renderLabel( None, "\n\n\n\n\n\n\n\n\n\n ", packing=[False, False] )
        separator.set_size_request( 250, 5 )
        #self.resultPageSeparators.append( separator )
        self.outputInfoBox.put( label, 20, base )
        self.outputInfoBox.put( separator, 20, base+20 )
        self.outputInfoBox.put( self.outputTestSummary, 30, base+30 )
    
    def set_text( self, text ):
        
        self.outputTestSummary.set_markup( text )

    def get_text( self ):
        
        return self.outputTestSummary.get_text( )

class OutputTestList( RenderingObject ):
    
    def __init__( self, handler, OutputArea ):
        
        RenderingObject.__init__( self )
        
        outputTestModel = gtk.ListStore( str,
                                         str,
                                       )

        self.outputTestView = self.pobject.renderTreeView( None, outputTestModel, 
                                               { "changed" : handler.get_callback("OUTPUT_TEST_TOGGLED") }, 
                                               gtk.SELECTION_SINGLE )
        self.pobject.renderScrolledWindow( OutputArea, self.outputTestView, gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC, gtk.SHADOW_NONE, gtk.CORNER_TOP_LEFT )

        self.pobject.renderTreeViewColumnText( self.outputTestView, _('Test'), 0, sort=True)
        self.pobject.renderTreeViewColumnText( self.outputTestView, _('Result'), 1, sort=True)
        
    def clear( self ):
        
        self.outputTestView.get_model( ).clear( )

    def append( self, record ):
        
        return self.outputTestView.get_model( ).append( record )

    def set_value( self, iter, num, message ):
        
        self.outputTestView.get_model().set_value( iter, num, message )

    def get_selected( self ):
        
        selection = self.outputTestView.get_selection()
        if selection != None: (model, iter) = selection.get_selected( )
        else: 
            iter = None
            model = None
            
        return model, iter
        
class StatusWindow( RenderingObject ):
    
    def __init__( self, handler, outputArea ):

        RenderingObject.__init__( self )
        self.StatusWindow = self.pobject.renderHBox( )

        align, self.progressBar = self.pobject.renderProgressBar( None, align=[0,5,5,5] )

        eventBox = self.pobject.renderEventBox( self.StatusWindow, events=gtk.gdk.BUTTON_PRESS_MASK, border=1, packing=[True, True] )
        eventBox.add( align )

        self.progressButton = self.pobject.renderButton( self.StatusWindow, "toggle", "gtk-goto-top", align=[0,0,5,0], 
                                                 callbacks={"toggled": handler.get_callback("PROGRESSBAR_SET_HIDE")} )
        self.progressButton.set_active( True )

        self.statusBar = gtk.Statusbar( )
        self.statusBar.add( self.StatusWindow )
        self.StatusWindow.hide( )

        self.statusBar.show( )
        outputArea.pack_start( self.statusBar, False, False, 0 )
    
    def show( self ):
        
        self.StatusWindow.show_all( )
        
    def hide( self ):
        
        self.StatusWindow.hide()
        
    def push_requirements( self, text ):
        
        frame = self.statusBar.get_children()[0]
        label_status = frame.get_children()[0]
        if label_status != None:
            str_req = ""
            for dep in text: str_req += dep+", "  
            str_req = str_req[:-2]
            self.statusBar.push( 1, "<b><span foreground='red'>"+_("Test does not meet the requirements: ")+str_req+"</span></b>" )
            label_status.set_use_markup(True)
        else: self.statusBar.push( 1, _("Test does not meet the requirements: ")+`text` )

    def pop( self, num ):
        
        self.statusBar.pop( num )

    def update_progress( self, progress=None, label=None ):
        """ Update progress and label in progressBar """

        if progress != None: 
            if progress > 1.0: progress = 1.0
            self.progressBar.set_fraction(progress)
            if progress == 1.0: self.progressBar.set_text( _("All tests done. ") )
            elif label != None: self.progressBar.set_text( `label` )
        
    def pb_is_active( self ):
        
        return self.progressButton.get_active( )
    
    def get_fraction( self ):
        
        return self.progressBar.get_fraction( )

class OutputTestResults( RenderingObject ):
    """
        Object with result lines from tests
    """
    
    def __init__( self, handler, outputArea ):
        
        RenderingObject.__init__( self )
        # list model - how the treeView will looks like
        outputModel = gtk.ListStore( gtk.gdk.Pixbuf,
                                    str,
                                    str,
                                    str,
                                    str,
                                  )
        
        # we want to filter lines in treeView
        self.messageModelFilter = outputModel.filter_new( )
        self.messageModelFilter.set_visible_func( self.filter_func )
        
        # we want to sort lines by columns
        #TODO: do my own sort function
        self.sortMessageModelFilter = gtk.TreeModelSort( self.messageModelFilter )

        # render outputView - gtk object
        self.outputView = self.pobject.renderTreeView( None, self.sortMessageModelFilter, 
                                               { "changed" : handler.get_callback("OUTPUT_TOGGLED") }, 
                                               gtk.SELECTION_SINGLE )
        # put it into scrolled window
        self.pobject.renderScrolledWindow( outputArea, self.outputView, gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC, gtk.SHADOW_NONE, gtk.CORNER_TOP_LEFT )

        self.pobject.renderTreeViewColumnPixbuf( self.outputView, _('Mark'), 0)
        self.pobject.renderTreeViewColumnText( self.outputView, _('Severity'), 1, sort=True )
        self.pobject.renderTreeViewColumnText( self.outputView, _('Test'), 2, sort=True )
        self.pobject.renderTreeViewColumnText( self.outputView, _('Num'), 3, visible=False)
        self.pobject.renderTreeViewColumnText( self.outputView, _('Description'), 4, expand=True )

        self.__filter = False

    def clear_model( self ):
        
        # TreeView -> TreeModelSort -> TreeModelFilter -> clear
        self.outputView.get_model().get_model().get_model().clear()

    def append_message( self, message ):
        """
        Append message to outputView

        @param message: message to append
        """

        if self.outputView.get_model() == None:
            self.__exception(_("Wrong call of appendMessage() function: missing gtk.TreeModel"), 50)
        if message == None:
            self.__exception(_("Wrong call of appendMessage() function: missing message string"), 50)
        
        # TreeView -> TreeModelSort -> TreeModelFilter -> append the message
        self.outputView.get_model().get_model().get_model().append( message )
        
    def filter_default( self, fmodel, fiter ):
        
             return not fmodel[fiter][1] == self.fUtils.iLabel

    def filter_func( self, fmodel, fiter ):
        """ 
        Function called to determine the visibility of a row 
        """
        
        if self.__filter == False: return self.filter_default( fmodel, fiter ) # nothing selected for now

        if self.__filter_info == False and fmodel[fiter][1] == self.fUtils.iLabel: return False
        elif self.__filter_iter != None and not self.__filter_all: 
            if self.__filter_test != self.fUtils.remove_mark( fmodel[fiter][2] ): return False
            else: return True
        else: return True

    def refilter( self, iter, filter_all, test_name, info ):
        
        self.__filter      = True
        self.__filter_iter = iter
        self.__filter_all  = filter_all
        self.__filter_test = test_name
        self.__filter_info = info
        self.messageModelFilter.refilter( )

class TestList( RenderingObject ):

    def __init__( self, handler, OutputArea ):

        """
        Rendering TreeView of tests
        """

        RenderingObject.__init__( self )
        #assert box == None, "RenderObject.TestView.box is None" #: debug box should not be None
        
        testModel = gtk.ListStore(  gobject.TYPE_STRING, 
                                gobject.TYPE_BOOLEAN, 
                                gobject.TYPE_BOOLEAN, 
                                gobject.TYPE_BOOLEAN, 
                                gobject.TYPE_BOOLEAN, 
                                gobject.TYPE_BOOLEAN,
                                gobject.TYPE_BOOLEAN,
                              )
        # make a popup menu       
        rootMenu = self.__init_popup_menu( handler, OutputArea )
        
        # continue with making treeView
        # TODO: make a method to connect signals for all objects
        self.testView = self.pobject.renderTreeView( None, testModel, 
                                             { "changed" : handler.get_callback("TESTVIEW_TOGGLED")}, 
                                             gtk.SELECTION_SINGLE)
        self.pobject.renderScrolledWindow( OutputArea, self.testView, gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC, gtk.SHADOW_NONE, gtk.CORNER_TOP_LEFT )
        
        self.pobject.renderTreeViewColumnText( self.testView, 'Test', 0, expand=True, sort=True )
        self.pobject.renderTreeViewColumnToggle( self.testView, '1', 1, {"toggled":handler.get_callback("TESTVIEW_LEVEL_TOGGLED")})
        self.pobject.renderTreeViewColumnToggle( self.testView, '2', 2, {"toggled":handler.get_callback("TESTVIEW_LEVEL_TOGGLED")})
        self.pobject.renderTreeViewColumnToggle( self.testView, '3', 3, {"toggled":handler.get_callback("TESTVIEW_LEVEL_TOGGLED")})
        self.pobject.renderTreeViewColumnToggle( self.testView, '4', 4, {"toggled":handler.get_callback("TESTVIEW_LEVEL_TOGGLED")})
        self.pobject.renderTreeViewColumnToggle( self.testView, '5', 5, {"toggled":handler.get_callback("TESTVIEW_LEVEL_TOGGLED")})

        self.testView.connect_object("button_press_event", handler.get_callback("TESTVIEW_BUTTON_PRESSED"), rootMenu.get_submenu(), self.testView )
        
    def __init_popup_menu( self, handler, OutputArea ):
        
        self.popupBar = self.renderMenuBar( OutputArea )
        self.popupBar.hide( )

        # Root menu "FILE"
        self.popupStartTest = self.pobject.renderMenuItem(_("Start the selected test"), 
                                         _("_Start the selected test"), 
                                         {"activate": handler.get_callback("START_TEST")}, 
                                         gtk.STOCK_REDO, 
                                         Note=_('Start the test'))
        self.popupStartLevel = self.pobject.renderMenuItem(_("Start checked tests in selected level"), 
                                          _("_Start checked tests in selected level"), 
                                          {"activate": handler.get_callback("START_LEVEL")}, 
                                          gtk.STOCK_REFRESH, 
                                          Note=_('Start the whole level'))
        self.popupLoadDefaultLevels = self.pobject.renderMenuItem(_("Load default level configuration"), 
                                          _("_Load default level configuration"), 
                                          {"activate": handler.get_callback("DEFAULT_LEVELS")}, 
                                          gtk.STOCK_UNDELETE, 
                                          Note=_('Load default configuration for levels'))
        self.popupRefreshTest = self.pobject.renderMenuItem(_("Clear and refresh test data"), 
                                          _("_Clear & refresh test data"), 
                                          {"activate": handler.get_callback("REFRESH_TEST")}, 
                                          gtk.STOCK_CLEAR, 
                                          Note=_('Removes any data owned by a test and runs it just to recreate them.'))
        space = gtk.SeparatorMenuItem()
        space.show()
        rootMenu = self.pobject.renderRootMenuItem( _("Root Menu"), _("_Root Menu"), self.popupBar, 
                                                    (self.popupStartTest, 
                                                     self.popupStartLevel, 
                                                     space,
                                                     self.popupLoadDefaultLevels, 
                                                     self.popupRefreshTest) )
        
        return rootMenu

    def set_level( self, level ):

        # clear 
        columns = self.testView.get_columns( )
        for col in columns:
            cell = col.get_cell_renderers()[0]
            cell.set_property( 'cell-background', 'white' )
            col.add_attribute( cell, 'cell_background_set', 6 )
            
        # set
        column = self.testView.get_column( level )
        cell = column.get_cell_renderers()[0]
        cell.set_property( 'cell-background', 'gray' )
        column.add_attribute( cell, 'cell_background_set', 6 )
        
    def clear( self ):
        
        self.testView.get_model().clear( )

    def get_all_tests( self ):
        
        model = self.testView.get_model()
        
        tests = []
        for test in model:
            tests.append( test )
            
        return [ test for test in model ]
        
    def get_selected_tests( self ):
        
        model, rows = self.testView.get_selection().get_selected_rows()
        test_list = [model.get_value( model.get_iter(test), 0 ) for test in rows]
        return test_list
    
    def get_active_tests( self, level ):
        
        model = self.testView.get_model()
        
        tests = []
        for test in model:
            if test[ level ]: tests.append( test ) 
        return tests
        
    def get_test_levels( self ):
        
        model = self.testView.get_model()

        testSuite = dict()
        
        for test in model:
            testSuite[ test[0] ] = []
            for i, level in enumerate( list(test)[1:-1] ):
                if level: testSuite[ test[0] ].append( i+1 )

        return testSuite

    def append_test( self, test ):
        
        self.testView.get_model().append( test )
        
    def set_test_levels( self, test, levels ):
        
        model = self.renderer.testView.get_model()

        for i, mtest in enumerate( model ):
            if mtest[0] == test:
                for level in [1,2,3,4,5]:
                    model[i][level] = level in levels
                    
    def select_test( self, num ):
        
        self.testView.get_selection().select_path( num )
        
    def set_popup_sensitivity( self, sensitive ):
        
        self.popupStartTest.set_sensitive( sensitive )
        self.popupStartLevel.set_sensitive( sensitive )
        self.popupLoadDefaultLevels.set_sensitive( sensitive )
        
class MenuBar( RenderingObject ):

    """
    Menu bar.. File, View, Edit, Help.. 

    @param box: box to pack in
    @type  box: C{gtk.HBox} or C{gtk.VBox}
    """
    
    def __init__( self, handler, OutputArea ):
        
        RenderingObject.__init__( self )
        
        #assert box == None, "RenderObject.MenuBar.box is None" #: debug box should not be None
        menuBar = self.renderMenuBar( OutputArea )
        
        # Root menu "FILE"
        self.start_test = self.pobject.renderMenuItem(_("Start the selected test"), 
                                         _("_Start the selected test"), 
                                         {"activate": handler.get_callback("START_TEST")}, 
                                         gtk.STOCK_REDO, 
                                         Note=_('Start the test'))
        self.start_level = self.pobject.renderMenuItem(_("Start checked tests in selected level"), 
                                          _("_Start checked tests in selected level"), 
                                          {"activate": handler.get_callback("START_LEVEL")}, 
                                          gtk.STOCK_REFRESH, 
                                          Note=_('Start the whole level'))
        start = self.pobject.renderRootMenuItem( _("Start"), _("_Start"), None, (self.start_test, self.start_level) )       

        separator = gtk.SeparatorMenuItem( )
        self.stop = self.pobject.renderMenuItem(_("Stop"), _("S_top"), 
                                   {"activate": handler.get_callback("STOP_TESTS")}, 
                                   gtk.STOCK_STOP, enabled=False)
        quit = self.pobject.renderMenuItem(_("Quit"), _("_Quit"), 
                                   {"activate": handler.get_callback("QUIT")}, 
                                   gtk.STOCK_QUIT, Note=_('Quit program'))
        self.pobject.renderRootMenuItem( _("File"), _("_File"), menuBar, (start, self.stop, separator, quit) )       
        
        # Root menu Edit
        #self.renderRootMenuItem( _("Edit"), _("_Edit"), menuBar, None )
        # Root menu View
        self.info = self.pobject.renderCheckMenuItem( _("Show info messages") )
        self.hint = self.pobject.renderCheckMenuItem( _("Show hints") )
        self.pobject.renderRootMenuItem( _("View"), _("_View"), menuBar, (self.info, self.hint) )
        # Root menu Tools
        #self.renderRootMenuItem( _("Tools"), _("_Tools"), menuBar, None )
        # Root menu Help
        about = self.pobject.renderMenuItem( _("About"), _("_About"), 
                                     {"activate": handler.get_callback("ABOUT_DIALOG")},
                                     gtk.STOCK_ABOUT, Note=_('About Dialog'))
        self.pobject.renderRootMenuItem( _("Help"), _("_Help"), menuBar, (about,) )

        self.hint.connect( "toggled", handler.get_callback("HINT_TOGGLED") )
        self.info.connect( "toggled", handler.get_callback("INFO_TOGGLED") )
        
    def get_info_active( self ):
        
        return self.info.get_active( )
    
    def get_hint_active( self ):
        
        return self.hint.get_active( )
    
    def set_info_active( self, active ):
        
        self.info.set_active( active )
        
    def set_hint_active( self, active ):
        
        self.hint.set_active( active )
        
    def set_sensitivity( self, sensitive ):
        
        self.start_test.set_sensitive( sensitive )
        self.start_level.set_sensitive( sensitive )
        self.stop.set_sensitive( not sensitive )

class ToolBar( RenderingObject ):
    
    """
     Bar with large buttons like Start, Stop, Quit, ...
     @param box: box to pack in
     @type  box: C{gtk.HBox} or C{gtk.VBox}
    """

    def __init__( self, handler, OutputArea ):

        RenderingObject.__init__( self )

        #assert box == None, "RenderObject.ToolBar.box is None" #: debug box should not be None

        toolBarBox = self.pobject.renderHBox( OutputArea )

        self.toolBar = self.pobject.renderToolBar( toolBarBox, packing=[True, True] )
        # ---
        self.toolStart = gtk.MenuToolButton( self.setIconStock( gtk.STOCK_REFRESH ), _("Start level") )
        self.toolStart.connect( "clicked", handler.get_callback("START_LEVEL") )
        self.toolStart.show ( )
        menu = gtk.Menu()
        menu.append( self.pobject.renderMenuItem( _("Start level"), 
                                          _("Start selected _level"), 
                                          {"activate": handler.get_callback("START_LEVEL")}, 
                                          gtk.STOCK_REFRESH, Note=_('Start whole level')) )
        menu.append( self.pobject.renderMenuItem( _("Start test"), 
                                          _("Start selected _test"), 
                                          {"activate": handler.get_callback("START_TEST")}, 
                                          gtk.STOCK_REDO, Note=_('Start choosen test')) )
 
        self.toolStart.set_menu( menu )
        # ---
        self.toolClean = gtk.MenuToolButton( self.setIconStock( gtk.STOCK_CLEAR ), _("Clear tests") )
        self.toolClean.connect( "clicked", handler.get_callback("REFRESH_TEST_ALL") )
        self.toolClean.show ( )
        menu = gtk.Menu()
        menu.append( self.pobject.renderMenuItem( _("Clear and refresh all tests"), 
                                          _("Clear & refresh all tests"), 
                                          {"activate": handler.get_callback("REFRESH_TEST_ALL")}, 
                                          gtk.STOCK_CLEAR, Note=_('Removes all data for all tests')) )
        menu.append( self.pobject.renderMenuItem( _("Clear and refresh selected test"), 
                                          _("Clear & refresh selected test"), 
                                          {"activate": handler.get_callback("REFRESH_TEST")}, 
                                          gtk.STOCK_CLEAR, Note=_('Removes any data owned by a test and runs it just to recreate them.')) )
 
        self.toolClean.set_menu( menu )
        # ---
        self.toolStop  = self.pobject.renderToolButton( _("Stop"), gtk.STOCK_STOP, {"clicked": handler.get_callback("STOP_TESTS")}, sensitive=False ) 
        self.toolInfo  = self.pobject.renderToggleToolButton( _("Show info"), gtk.STOCK_INFO, {"clicked": handler.get_callback("INFO_TOGGLED")} ) 
        self.toolHints = self.pobject.renderToggleToolButton( _("Show hints"), gtk.STOCK_DIALOG_QUESTION, {"clicked": handler.get_callback("HINT_TOGGLED")} ) 
        self.toolQuit  = self.pobject.renderToolButton( _("Quit"), gtk.STOCK_QUIT, {"clicked": handler.get_callback("QUIT")} ) 
        self.toolSave  = self.pobject.renderToolButton( _("Save..."), gtk.STOCK_SAVE, {"clicked": handler.get_callback("SAVE_OUTPUT")} ) 

        self.toolBar.insert( self.toolStart, -1 )
        self.toolBar.insert( self.toolStop, -1 )
        space = gtk.SeparatorToolItem( )
        space.show()
        self.toolBar.insert( space, -1 )
        self.toolBar.insert( self.toolClean, -1 )
        space = gtk.SeparatorToolItem( )
        space.show()
        self.toolBar.insert( space, -1 )
        self.toolBar.insert( self.toolSave, -1 )
        space = gtk.SeparatorToolItem( )
        space.show()
        self.toolBar.insert( space, -1 )
        self.toolBar.insert( self.toolInfo, -1 )
        self.toolBar.insert( self.toolHints, -1 )
        space = gtk.SeparatorToolItem( )
        space.show()
        self.toolBar.insert( space, -1 )
        self.toolBar.insert( self.toolQuit, -1 )

        self.pobject.renderAnimation( toolBarBox, "/usr/share/pixmaps/sectool-min.png", align=[5, 5, 0, 20] )
        
    def set_sensitive( self, sensitive ):
        
        self.toolStop.set_sensitive( sensitive )
        self.toolSave.set_sensitive( not sensitive )
        self.toolStart.set_sensitive( not sensitive )
        
    def set_info_active(self, active ):
        
        self.toolInfo.set_active( active )
        
    def set_hint_active(self, active ):
        
        self.toolHints.set_active( active )
        
